@@ -58,7 +58,7 @@ class Config( object ):
     __metaclass__ = MetaSingleton
     
     def __init__(self):
-            self.__version = '1.5.1'
+            self.__version = '1.5.3'
             import Local.Config.Config
             import Local.Config.Execution
             #############################
@@ -164,7 +164,7 @@ class Config( object ):
             self._particular_debug = {}
             
             self._accounting = False
-            self._session_debug = False
+            self._session_debug = None
             self._status_debug = False
             
             if self._htdocs_prefix:
@@ -1169,7 +1169,7 @@ class Config( object ):
         elif 'DEFAULT' in self._module_load:
             module_load = self._module_load['DEFAULT']
         else:
-            #module_load is not define ifor this interface and there is no default
+            #module_load is not defined for this interface and there is no default
             #don't use module load
             module_load = None
         if module_load is not None:
