@@ -127,7 +127,7 @@ class MobyleJob:
                     raise UserValueError( msg = resp.user_message )                 
             else:
                 self.userEmail = None
-            email_notify = email_notify.lower()
+            email_notify = email_notify.lower()        
             resp =  checkHost()
             if not resp.status:
                 msg = "%s %s %s %s %s" %( jobName,
@@ -226,7 +226,130 @@ class MobyleJob:
         self.session = session
         if self._job is not None:
             self._job.setSession( session )
-        
+    
+    
+    def _toFileAndConvert(self, parameter , value):
+        paramName = parameter.getName()
+        if len( value ) != 5 : 
+            msg = "Problem with the infile : " + paramName
+            self._logError( logMsg = msg ,
+                            userMsg = "Internal Mobyle Server Error"
+                          )
+            raise UserValueError( parameter = parameter , msg = msg )
+        else:    
+            
+            destFileName , data  , src , srcFileName, idx = value
+            if not destFileName and not data and not src:
+                data  = None
+                src = None
+            elif destFileName and not ( data or src ):
+                msg = "parameter %s: is empty. please check your data" % paramName
+                self._logError( logMsg = msg ,
+                                userMsg = msg
+                                )
+                raise UserValueError( parameter =  parameter , msg = msg )
+            elif data and src:
+                msg = "parameter %s: you cannot specify data and source at the same time" % paramName
+                self._logError( logMsg = msg ,
+                                userMsg = msg
+                                )
+                raise UserValueError( parameter =  parameter , msg = msg )
+
+            elif not srcFileName and not data and src:
+                msg = "parameter %s: if you specify a src, you must specify a src file name" % paramName
+                self._logError( logMsg = msg ,
+                                userMsg = msg
+                                )
+                raise UserValueError( parameter =  parameter , msg = msg )
+               
+            elif not srcFileName and data and not src:
+                srcFileName = paramName + ".data"      
+                                
+            # I put systematically the value in index.xml even if its the
+            # same as the previous value because the value is a filename 
+            # and 2 files with the same name could have different contents
+            
+            ##fileName = basename
+            acceptedMobyleType = parameter.getType()
+            dataType = acceptedMobyleType.getDataType()
+            paramName = parameter.getName()
+            try:
+                fileName, sizeIn = dataType.toFile( data, self._job , destFileName, src , srcFileName )
+            except MobyleError, err:
+                msg = "%s/%s : %s : %s"%( self._service.getName(), self.getJobid() , paramName, err)
+                self.m_log.error( msg )
+                raise MobyleError( msg )
+            detectedMobyleType = acceptedMobyleType.detect( ( self._job, fileName ) )
+            acceptedFormats = acceptedMobyleType.getAcceptedFormats()
+            if detectedMobyleType.dataFormat is None :
+                self.jobState.addInputDataFile( paramName, ( fileName  ,  sizeIn ,  'not detected' ) )
+                self.jobState.commit()
+                if not acceptedFormats:
+                    return fileName     
+                elif ( set( [ f for f , r in acceptedFormats] )- set( dataType.supportedFormat() ) ):
+                    # format accepted by the parameter  -  format supported by the conevrters
+                    # case of a not detected format but the parameter accept some formats not supported by converter
+                    self.m_log.warning("%s/%s : %s : a data format which has not been detected has been accepted in job" %( self._service.getName(), self.getJobid() , paramName ))
+                    return fileName 
+                else:
+                    # case of a not detected format and the all format accepetd by the parameter are supported by the converters 
+                    msg = "Your %s format cannot be detected: please submit your data in one of the following formats: %s"%( dataType.getName() , dataType.supportedFormat() )
+                    sm = StatusManager()
+                    sm.setStatus( self.getDir() , Status( code= 5 , message= msg ))
+                    raise UserValueError( parameter = parameter ,
+                                          msg = msg )
+            else: #the format of the data has been detected
+                outFileName = None
+                if not acceptedFormats:#if there isn't any accepted format => all formats are available
+                    self.jobState.addInputDataFile( paramName ,
+                                                            ( fileName  ,  sizeIn ,  detectedMobyleType.dataFormat ) ,
+                                                            fmtProgram = detectedMobyleType.format_program
+                                                            )
+                    self.jobState.commit()
+                    return fileName 
+                    if self._debug > 1:
+                            self.build_log.debug("self._service.setValue( %s , %s )" %( paramName , fileName ) )
+                else:# some accepted format are specified
+                    for out_format , force_reformat in acceptedFormats:
+                        if detectedMobyleType.dataFormat == out_format:
+                            if force_reformat:
+                                continue
+                            else:
+                                self.jobState.addInputDataFile( paramName ,
+                                                                ( fileName  ,  sizeIn ,  detectedMobyleType.dataFormat ) ,
+                                                                fmtProgram = detectedMobyleType.format_program
+                                                                )
+                                self.jobState.commit()
+                                outFileName = fileName
+                                break
+                        else:
+                            continue
+                    if outFileName is None:#the data must be converted
+                        try:
+                            outFileName , convertedMobyleType = detectedMobyleType.convert( ( self._job, fileName ) , acceptedMobyleType )
+                            sizeOut = os.path.getsize( os.path.join( self._job.getDir(),outFileName ) )
+                            self.jobState.addInputDataFile( paramName ,
+                                                            ( fileName  ,  sizeIn ,  detectedMobyleType.dataFormat ) ,
+                                                            fmtProgram = convertedMobyleType.format_program ,
+                                                            formattedFile = ( outFileName , sizeOut , convertedMobyleType.dataFormat )
+                                                            )
+                            self.jobState.commit()
+                        except UnSupportedFormatError ,err :
+                            self.m_log.error( "UnSupportedFormatError:%s"%err ,exc_info=True)
+                            msg = str( err )
+                            sm = StatusManager()
+                            sm.setStatus( self.getDir() , Status( code= 5 , message= msg ))
+                            raise UserValueError( parameter = parameter , msg = msg )
+                        except Exception, err:
+                            self.m_log.error( str(err), exc_info= True)
+                   
+                return outFileName
+
+            
+                        
+                        
+                        
+                        
     def setValue( self , paramName , value ):
         """
         set the value for one parameter. the value is cast in the right type. if the value cannot be cast a UserValueError is raised
@@ -260,122 +383,28 @@ class MobyleJob:
             if value == '' :
                 value = None
             if self._service.isInfile( paramName ):
-                if len( value ) == 4 : ###MobyleJob is called from a cgi
-                    destFileName , data  , src , srcFileName = value
-                    if not destFileName and not data and not src:
-                        data  = None
-                        src = None
-                    elif destFileName and not ( data or src ):
-                        msg = "parameter %s: is empty. please check your data" % paramName
-                        self._logError( logMsg = msg ,
-                                        userMsg = msg
-                                        )
-                        raise UserValueError( parameter =  parameter , msg = msg )
-                    elif data and src:
-                        msg = "parameter %s: you cannot specify data and source at the same time" % paramName
-                        self._logError( logMsg = msg ,
-                                        userMsg = msg
-                                        )
-                        raise UserValueError( parameter =  parameter , msg = msg )
-
-                    elif not srcFileName and not data and src:
-                        msg = "parameter %s: if you specify a src, you must specify a src file name" % paramName
-                        self._logError( logMsg = msg ,
-                                        userMsg = msg
-                                        )
-                        raise UserValueError( parameter =  parameter , msg = msg )
-                       
-                    elif not srcFileName and data and not src:
-                        srcFileName = paramName + ".data"      
-
-                                        
-                    # I put systematically the value in index.xml even if its the
-                    # same as the previous value because the value is a filename 
-                    # and 2 files with the same name could have different contents
-                    
-                    ##fileName = basename
-                    try:
-                        fileName, sizeIn = dataType.toFile( data, self._job , destFileName, src , srcFileName )
-                    except MobyleError, err:
-                        msg = "%s/%s : %s : %s"%( self._service.getName(), self.getJobid() , paramName, err)
-                        self.m_log.error( msg )
-                        raise MobyleError( msg )
-                    detectedMobyleType = acceptedMobyleType.detect( ( self._job, fileName ) )
-                    acceptedFormats = acceptedMobyleType.getAcceptedFormats()
-                    if detectedMobyleType.dataFormat is None :
-                        self.jobState.setInputDataFile( paramName ,
-                                                        ( fileName  ,  sizeIn ,  'not detected' ) ,
-                                                        )
-                        self.jobState.commit()
-                        
-                        if not acceptedFormats:
-                            parameter.setValue( fileName )    
-                        elif ( set( [ f for f , r in acceptedFormats] )- set( dataType.supportedFormat() ) ):
-                            # format accepted by the parameter  -  format supported by the conevrters
-                            # case of a not detected format but the parameter accept some formats not supported by converter
-                            self.m_log.warning("%s/%s : %s : a data format which has not been detected has been accepted in job" %( self._service.getName(), self.getJobid() , paramName ))
-                            parameter.setValue( fileName )
-                        else:
-                            # case of a not detected format and the all format accepetd by the parameter are supported by the converters 
-                            msg = "Your %s format cannot be detected: please submit your data in one of the following formats: %s"%( dataType.getName() , dataType.supportedFormat() )
-                            sm = StatusManager()
-                            sm.setStatus( self.getDir() , Status( code= 5 , message= msg ))
-                            raise UserValueError( parameter = parameter ,
-                                                  msg = msg )
-                    else: #the format of the data has been detected
-                        outFileName = None
-                        if not acceptedFormats:#if there isn't any accepted format => all formats are available
-                            self.jobState.setInputDataFile( paramName ,
-                                                                    ( fileName  ,  sizeIn ,  detectedMobyleType.dataFormat ) ,
-                                                                    fmtProgram = detectedMobyleType.format_program
-                                                                    )
-                            self.jobState.commit()
-                            parameter.setValue( fileName )
-                            if self._debug > 1:
-                                    self.build_log.debug("self._service.setValue( %s , %s )" %( paramName , fileName ) )
-                        else:# some accepted format are specified
-                            for out_format , force_reformat in acceptedFormats:
-                                if detectedMobyleType.dataFormat == out_format:
-                                    if force_reformat:
-                                        continue
-                                    else:
-                                        self.jobState.setInputDataFile( paramName ,
-                                                                        ( fileName  ,  sizeIn ,  detectedMobyleType.dataFormat ) ,
-                                                                        fmtProgram = detectedMobyleType.format_program
-                                                                        )
-                                        self.jobState.commit()
-                                        outFileName = fileName
-                                        break
-                                else:
-                                    continue
-                            
-                            if outFileName is None:#the data must be converted
-                                try:
-                                    outFileName , convertedMobyleType = detectedMobyleType.convert( ( self._job, fileName ) , acceptedMobyleType )
-                                    
-                                    sizeOut = os.path.getsize( os.path.join( self._job.getDir(),outFileName ) )
-                                    self.jobState.setInputDataFile( paramName ,
-                                                                    ( fileName  ,  sizeIn ,  detectedMobyleType.dataFormat ) ,
-                                                                    fmtProgram = convertedMobyleType.format_program ,
-                                                                    formattedFile = ( outFileName , sizeOut , convertedMobyleType.dataFormat )
-                                                                    )
-                                    self.jobState.commit()
-                                except UnSupportedFormatError ,err :
-                                    msg = str( err )
-                                    sm = StatusManager()
-                                    sm.setStatus( self.getDir() , Status( code= 5 , message= msg ))
-                                    raise UserValueError( parameter = parameter , msg = msg )
-                            parameter.setValue( outFileName )
-                            if self._debug > 1:
-                                    self.build_log.debug("self._service.setValue( %s , %s )" %( paramName , outFileName ) )
-
-                else:
-                        msg = "Problem with the infile : " + paramName
-                        self._logError( logMsg = msg ,
-                                        userMsg = "Internal Mobyle Server Error"
-                                        )
-                        raise UserValueError( parameter = parameter , msg = msg )
-                    
+                
+                ###############################################################
+                if dataType.isMultiple():
+                    fileNames = []
+                    value.sort(key=lambda item:item[4])
+                    for item in value:
+                        #destFileName , data  , src , srcFileName, idx = item
+                        #destFileName file name in the destination folder
+                        #data file contents (if src is None)
+                        #src: folder where the source file can be found (if data is None)
+                        #srcFileName: file name in the src folder (if data is None)
+                        #idx: index in the ordered list of values
+                        #self.m_log.debug( "destFileName=%s ,data=%s  , src=%s , srcFileName=%s"% (destFileName , data  , src , srcFileName))
+                        fileName = self._toFileAndConvert( parameter , item )
+                        fileNames.append( fileName )
+                    parameter.setValue( fileNames )
+                    if self._debug > 1:
+                        self.build_log.debug("self._service.setValue( %s , %s )" %( paramName , fileNames ) )
+                else:#parameter is represents One file
+                    fileName = self._toFileAndConvert( parameter , value )
+                    parameter.setValue( fileName )
+                   
             else : #the parameter is not an infile
                 oldValue = parameter.getValue()
                 try:
@@ -387,7 +416,7 @@ class MobyleJob:
                             self.build_log.debug( "self._service.setValue( %s , %s )" %( paramName , converted_value ) )
                         except:
                             self.build_log.debug( "!!!!!!!! problem during reporting log !!!!!!!!!!!!" )
-                except UserValueError, err:
+                except UserValueError , err:
                         msg =  str( err )
                         self._logError( logMsg = msg ,userMsg = msg)
                         raise UserValueError(parameter=parameter, msg=msg)
@@ -589,7 +618,7 @@ class MobyleJob:
                         continue #next parameter
             unixMasks = self._service.getFilenames( paramName , proglang = 'python' )
             service_name = self._service.getName()
-            unixMasks += ["%s.out"%service_name , "%s.err"%service_name ]
+            #unixMasks += ["%s.out"%service_name , "%s.err"%service_name ]
             allMasks += unixMasks 
             workdir = self.getDir()
             for mask in unixMasks:
@@ -601,22 +630,35 @@ class MobyleJob:
                                                                                                                            mask
                                                                                                                            ) )
                     potCollisions += [ os.path.basename( f ) for f in  Files ]
-
-        infiles = self._service.getAllInFileParameter()            
+        infiles = self._service.getAllInFileParameter() 
         for infileName in potCollisions:
             for parameter in infiles:
-                oldName = parameter.getValue()
-                if oldName == infileName:
-                    newName = "mob_%s_collision" %( infileName )
-                    parameter.renameFile(  workdir , newName  )
-                    self.jobState.renameInputDataFile( parameter.getName() , newName )
-                    self.jobState.commit()
-                    self.m_log.warning( '%s/%s : Potential Collision : %s file was renamed into %s' %( self._service.getName(),
-                                                                                                      self._job.getKey() ,
-                                                                                                      oldName ,
-                                                                                                      newName )
-                    )
-        
+                oldNames = self.jobState.getInput( parameter.getName())
+                oldNames = [ fileName for fileName , size , fmt in oldNames ]
+                
+                for oldName in oldNames:
+                    if oldName == infileName:
+                        newName = "mob_%s_collision" %( infileName )
+                        parameter.renameFile(  workdir , oldName, newName  )
+                        values = parameter.getValue()
+                        sep = parameter.getSeparator()
+                        if sep is not None:
+                            values = values.split( sep )
+                            if oldName in values: #the collision can be between input before conversion and output so the oldname is not in evaluator (seq.out -> seq.fasta) 
+                                oldName_i = values.index( oldName )
+                                values[ oldName_i ] = newName
+                                parameter.setvalue( values )
+                        else:
+                            if oldName == values:
+                                 parameter.setValue( newName )
+                        self.jobState.renameInputDataFile( parameter.getName() , oldName, newName )
+                        self.jobState.commit()
+                        self.m_log.warning( '%s/%s : Potential Collision : %s file was renamed into %s' %( self._service.getName(),
+                                                                                                          self._job.getKey() ,
+                                                                                                          oldName ,
+                                                                                                          newName )
+                        )
+            
         for mask in allMasks :
             mask = os.path.join( workdir , mask )
             collideFiles = glob.glob( mask )
@@ -631,7 +673,6 @@ class MobyleJob:
                                 )
                 raise MobyleError , msg
 
-
     def run(self):
         """
         run a job
