@@ -193,13 +193,13 @@ class EmailAddress:
             except:
                 return NetResponse( self.INVALID , "no mail server" , userMsg , method )
             return NetResponse( self.CONTINUE , '' , '' , method )
-        except dns.name.EmptyLabel:
+        except (dns.name.EmptyLabel, dns.resolver.NoNameservers):
             return NetResponse( self.INVALID , "no domain name server" , userMsg , method )
         except dns.exception.Timeout:
             return NetResponse( self.CONTINUE, "dns timeout" , userMsg , method )
         except Exception, err:
             msg = "unexpected error in  Email._checkDns : "+ str( err )
-            n_log.critical( msg )
+            n_log.critical( msg, exc_info = True )
             return NetResponse( self.INVALID , msg , userMsg , method )
         return NetResponse( self.CONTINUE , '' , '' , method )
             
