@@ -1,7 +1,7 @@
 ########################################################################################
 #                                                                                      #
 #   Author: Bertrand Neron,                                                            #
-#   Organization:'Biological Software and Databases' Group, Institut Pasteur, Paris.   #  
+#   Organization:'Biological Software and Databases' Group, Institut Pasteur, Paris.   #
 #   Distributed under GPLv2 Licence. Please refer to the COPYING.LIB document.         #
 #                                                                                      #
 ########################################################################################
@@ -16,7 +16,7 @@ os.environ['MOBYLEHOME'] = MOBYLEHOME
 
 if ( MOBYLEHOME ) not in sys.path:
     sys.path.append( MOBYLEHOME )
-if ( os.path.join( MOBYLEHOME , 'Src' ) ) not in sys.path:    
+if ( os.path.join( MOBYLEHOME , 'Src' ) ) not in sys.path:
     sys.path.append( os.path.join( MOBYLEHOME , 'Src' ) )
 
 import Mobyle.Test.MobyleTest
@@ -32,9 +32,9 @@ DATADIR = os.path.dirname( __file__ )
 
 class TransactionTest(unittest.TestCase):
     """Tests the functionalities of Transaction"""
-    
+
     def setUp(self):
-        """ 
+        """
         setting up the configuration for the test, including:
         - test configuration that does not check dns or try to send confirmation emails
         - test session email
@@ -54,7 +54,7 @@ class TransactionTest(unittest.TestCase):
         self.xmlSourcePath = os.path.join( DATADIR , 'session_test.xml')
         self.sessionPath = os.path.join( self.sessionDir , Session.FILENAME )
         self.dataID = [ "f835ddf28cb6aa776a41d7c71ec9cb5a.out" , "16bd6f89e732eb7cb0bbc13c65d202d3.aln" ]
-        
+
         dtf = DataTypeFactory()
         dataType1   = dtf.newDataType( 'Sequence' )
         mobyleType1 = MobyleType( dataType1 , bioTypes = [ 'Protein' , 'Nucleic' ] , dataFormat =  'SWISSPROT' )
@@ -67,7 +67,7 @@ class TransactionTest(unittest.TestCase):
                         'dataBegin'    :  'ID   104K_THEPA              Reviewed;         924' ,
                         'inputModes' : [ 'result' ] ,
                         'producedBy' : [ 'file://tmp/mobyle/results/dnadist/G25406684175968' ] ,
-                        'usedBy'     : [] 
+                        'usedBy'     : []
                         } ,
                         {'dataName'  : '16bd6f89e732eb7cb0bbc13c65d202d3.aln' ,
                          'userName'   : 'clustal.aln' ,
@@ -77,10 +77,10 @@ class TransactionTest(unittest.TestCase):
                          'inputModes' : [ 'upload' , 'paste' ] ,
                          'producedBy' : [] ,
                          'usedBy'     : [ 'file://tmp/mobyle/results/dnadist/G25406684175968' ,
-                                         'file://tmp/mobyle/results/dnadist/X25423048243999' ] 
+                                         'file://tmp/mobyle/results/dnadist/X25423048243999' ]
                          }
-                        ]        
-        
+                        ]
+
         self.jobID = [ "file://tmp/mobyle/results/dnadist/X25423048243999" ,
                        "file://tmp/mobyle/results/dnadist/G25406684175968" ]
         self.jobs  = [ {'jobID'       : 'file://tmp/mobyle/results/dnadist/X25423048243999' ,
@@ -100,7 +100,7 @@ class TransactionTest(unittest.TestCase):
                        'dataUsed'    : ['16bd6f89e732eb7cb0bbc13c65d202d3.aln'] ,
                        }
                       ]
-    
+
     def tearDown(self):
         shutil.rmtree( self.cfg.test_dir , ignore_errors= True )
 
@@ -121,24 +121,24 @@ class TransactionTest(unittest.TestCase):
         #test anonymous session without email
         Transaction.create( self.sessionPath , False , False )
         os.unlink( self.sessionPath )
-        
+
         #test anonymous session with passwd
         self.assertRaises( SessionError , Transaction.create , self.sessionPath , False , False , passwd = self.passwd )
-        
+
         #test anonymous session with email
         Transaction.create( self.sessionPath , False , False , userEmail = self.email )
         os.unlink( self.sessionPath )
-        
+
         #test authenticated session without passwd -> SessionError
         self.assertRaises( SessionError , Transaction.create , self.sessionPath , True , False , userEmail = self.email )
-        
+
         #test authenticated session without email -> SessionError
         self.assertRaises( SessionError , Transaction.create , self.sessionPath , True , False , passwd = self.passwd )
-        
+
         #test authenticated session with email and passwd
         Transaction.create( self.sessionPath , True , False , userEmail = self.email , passwd = self.passwd )
-        os.unlink( self.sessionPath ) 
-        
+        os.unlink( self.sessionPath )
+
         #test authenticated session with activatingKey
         Transaction.create( self.sessionPath , True , False , activatingKey = self.actkey , userEmail = self.email , passwd = self.passwd )
         os.unlink( self.sessionPath )
@@ -147,7 +147,7 @@ class TransactionTest(unittest.TestCase):
         Transaction.create( self.sessionPath , False , False )
         self.assertRaises( SessionError , Transaction.create , self.sessionPath , False , False )
         os.unlink( self.sessionPath )
-        
+
     def testCommit(self):
         Transaction.create( self.sessionPath , False , False )
 
@@ -178,7 +178,7 @@ class TransactionTest(unittest.TestCase):
         shutil.copy( self.xmlSourcePath , self.sessionPath )
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
         transaction.removeData( self.dataID[0] )
-        self. assertFalse( transaction.hasData(self.dataID[0]  ) )
+        self. assertFalse( transaction.hasData(self.dataID[0] ) )
         transaction.rollback()
         transaction = Transaction( self.sessionPath , Transaction.READ )
         self.assertTrue( transaction.hasData( self.dataID[0] ) )
@@ -197,7 +197,7 @@ class TransactionTest(unittest.TestCase):
             os.execv( cmd[0] , cmd )
         ## sub-process end
         transaction.commit()
-        
+
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
         ## sub-process start
         childPid = os.fork()
@@ -209,14 +209,14 @@ class TransactionTest(unittest.TestCase):
             os.execv( cmd[0] , cmd )
         ## sub-process end
         transaction.commit()
-        
+
     def testgetID(self):
         Transaction.create( self.sessionPath , False , False )
         transaction = Transaction( self.sessionPath , Transaction.READ )
         receivedID = transaction.getID()
         transaction.commit()
         self.assertEqual( receivedID , self.sessionKey )
-        
+
     def testEmail(self):
         Transaction.create( self.sessionPath , False , False )
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
@@ -226,7 +226,7 @@ class TransactionTest(unittest.TestCase):
         receivedEmail = transaction.getEmail()
         transaction.commit()
         self.assertTrue( receivedEmail == self.email )
-        
+
     def testisActivated(self):
 	# Test unactivated session
         Transaction.create( self.sessionPath , False , False )
@@ -265,24 +265,24 @@ class TransactionTest(unittest.TestCase):
         active = transaction.isActivated()
         self.assertFalse( active )
         os.unlink( self.sessionPath )
-        
-        
+
+
     def testActivatingKey(self):
         Transaction.create( self.sessionPath , False , True , activatingKey = self.actkey , userEmail = None)
         transaction = Transaction( self.sessionPath , Transaction.READ )
         actKey = transaction.getActivatingKey()
         transaction.commit()
         self.assertTrue( actKey == self.actkey )
-        
-        
+
+
     def testAuthenticated(self):
         Transaction.create( self.sessionPath , True , True , activatingKey = self.actkey , userEmail = self.email , passwd = self.passwd )
         transaction = Transaction( self.sessionPath , Transaction.READ )
         auth = transaction.isAuthenticated()
         transaction.commit()
-        self.assertTrue( auth )   
-    
-    
+        self.assertTrue( auth )
+
+
     def testPasswd(self):
         Transaction.create( self.sessionPath , False , False )
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
@@ -293,7 +293,7 @@ class TransactionTest(unittest.TestCase):
         receivedPasswd =  transaction.getPasswd()
         transaction.commit()
         self.assertTrue( sendPasswd == receivedPasswd )
-          
+
 
     def testCaptchaSolution( self ):
         sendSoluce = 'solution'
@@ -319,9 +319,9 @@ class TransactionTest(unittest.TestCase):
     def testHasData(self):
         shutil.copy( self.xmlSourcePath , self.sessionPath)
         transaction = Transaction( self.sessionPath , Transaction.READ )
-        self.assertTrue( transaction.hasData( self.dataID[1] )  )      
+        self.assertTrue( transaction.hasData( self.dataID[1] ) )
         transaction.commit()
-        
+
     def testRenameData( self ):
         shutil.copy( self.xmlSourcePath , self.sessionPath)
         newUserName = '__new_user_name_for_the_data__'
@@ -340,8 +340,8 @@ class TransactionTest(unittest.TestCase):
         transaction.commit()
         self.assertTrue( len( receivedData ) == 2 )
         for data in receivedData:
-            self.assertTrue( data in self.datas ) 
-        
+            self.assertTrue( data in self.datas )
+
     def testRemoveData( self ):
         shutil.copy( self.xmlSourcePath , self.sessionPath )
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
@@ -350,7 +350,7 @@ class TransactionTest(unittest.TestCase):
         transaction = Transaction( self.sessionPath , Transaction.READ )
         self.assertFalse( transaction.hasData( self.dataID[0] ) )
         transaction.commit()
-        
+
     def testLinkJobInput2Data( self ):
         Transaction.create( self.sessionPath , False , True )
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
@@ -361,8 +361,8 @@ class TransactionTest(unittest.TestCase):
                                 sendData[ 'Type' ] ,
                                 sendData[ 'dataBegin' ] ,
                                 sendData[ 'inputModes' ] ,
-                                producedBy = [] ,  
-                                usedBy = [] 
+                                producedBy = [] ,
+                                usedBy = []
                                 )
         sendJob = self.jobs[ 0 ]
         transaction.createJob( sendJob['jobID'] ,
@@ -372,7 +372,7 @@ class TransactionTest(unittest.TestCase):
                                 sendJob['date'] ,
                                 [] ,
                                 []
-                                )        
+                                )
         transaction.commit()
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
         transaction.linkJobInput2Data( [ sendData[ 'dataName' ] ] , [ sendJob['jobID'] ])
@@ -394,8 +394,8 @@ class TransactionTest(unittest.TestCase):
                                 sendData[ 'Type' ] ,
                                 sendData[ 'dataBegin' ] ,
                                 sendData[ 'inputModes' ] ,
-                                producedBy = [] ,  
-                                usedBy = [] 
+                                producedBy = [] ,
+                                usedBy = []
                                 )
         sendJob = self.jobs[ 0 ]
         transaction.createJob( sendJob['jobID'] ,
@@ -405,7 +405,7 @@ class TransactionTest(unittest.TestCase):
                                 sendJob['date'] ,
                                 [] ,
                                 []
-                                )        
+                                )
         transaction.commit()
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
         transaction.linkJobOutput2Data( [ sendData[ 'dataName' ] ] , [ sendJob['jobID'] ])
@@ -416,8 +416,8 @@ class TransactionTest(unittest.TestCase):
         transaction.commit()
         self.assertEqual( data[ 'producedBy' ][0] , sendJob[ 'jobID' ] )
         self.assertEqual( job[ 'dataProduced' ][0] , sendData[ 'dataName' ] )
-        
-        
+
+
     def testAddInputModes(self ):
         shutil.copy( self.xmlSourcePath , self.sessionPath )
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
@@ -439,7 +439,7 @@ class TransactionTest(unittest.TestCase):
         transaction.commit()
         data[ 'inputModes' ].sort()
         self.assertEqual( data[ 'inputModes' ] , result )
-        
+
     def testCreateAndGetData( self ):
         Transaction.create( self.sessionPath , True , True , activatingKey = 'uneJolieCle' , userEmail = self.email , passwd = self.passwd )
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
@@ -450,25 +450,27 @@ class TransactionTest(unittest.TestCase):
                                 sendData[ 'Type' ] ,
                                 sendData[ 'dataBegin' ] ,
                                 sendData[ 'inputModes' ] ,
-                                producedBy = sendData[ 'producedBy' ] ,  
-                                usedBy = sendData[ 'usedBy' ] 
+                                producedBy = sendData[ 'producedBy' ] ,
+                                usedBy = sendData[ 'usedBy' ]
                                 )
         transaction.commit()
         transaction = Transaction( self.sessionPath , Transaction.READ )
         receivedData = transaction.getData( sendData[ 'dataName' ] )
         transaction.commit()
         self.assertTrue( receivedData == sendData )
-        
+
     ####################
     #
     #   jobs methods
     #
     ####################
-    
+
     def testHasJob( self ):
         shutil.copy( self.xmlSourcePath , self.sessionPath )
+        jobid = "file://tmp/mobyle/results/dnadist/G25406684175968"
         transaction = Transaction( self.sessionPath , Transaction.READ )
-        self.assertTrue( transaction.hasJob( "file://tmp/mobyle/results/dnadist/G25406684175968" ) )
+        self.assertTrue( transaction.hasJob( jobid ) )
+        self.assertFalse( transaction.hasJob( jobid.replace('G','Z') ) )
         transaction.commit()
 
     def testGetAllJobs( self ):
@@ -480,7 +482,7 @@ class TransactionTest(unittest.TestCase):
 
         for job in receivedJobs:
             self.assertTrue( job in self.jobs )
-            
+
     def testCreateAndGetJob( self ):
         Transaction.create( self.sessionPath , False , True )
         sendJob = self.jobs[ 1 ]
@@ -498,17 +500,17 @@ class TransactionTest(unittest.TestCase):
         receivedJob = transaction.getJob( sendJob[ 'jobID' ] )
         transaction.commit()
         self.assertEqual( sendJob , receivedJob )
-        
-        
+
+
     def testRemoveJob( self ):
         shutil.copy( self.xmlSourcePath , self.sessionPath )
         transaction = Transaction( self.sessionPath , Transaction.WRITE )
-        transaction.removeJob( self.jobID[1] )  
+        transaction.removeJob( self.jobID[1] )
         transaction.commit()
         transaction = Transaction( self.sessionPath , Transaction.READ )
         self.assertFalse( transaction.hasJob( self.jobID[1] ) )
         transaction.commit()
-           
+
     def testRenameJob( self ):
         shutil.copy( self.xmlSourcePath , self.sessionPath )
         newUserName = "mon beau job"
@@ -519,7 +521,7 @@ class TransactionTest(unittest.TestCase):
         job = transaction.getJob( self.jobID[1] )
         transaction.commit()
         self.assertEqual( newUserName , job[ 'userName' ] )
-        
+
     def testUpdateJobStatus( self ):
         shutil.copy( self.xmlSourcePath , self.sessionPath )
         sendStatus = Mobyle.Status.Status( code = 7 ) #hold
@@ -531,5 +533,78 @@ class TransactionTest(unittest.TestCase):
         transaction.commit()
         self.assertEqual( sendStatus , job[ 'status' ] )
         
+    def testSetAndGetJobLabel(self):
+        shutil.copy( self.xmlSourcePath , self.sessionPath )
+        #there is not labels yet in jobs
+        transaction = Transaction( self.sessionPath , Transaction.READ )
+        label_recieved = transaction.getJobLabels(self.jobID[0])
+        transaction.commit()
+        self.assertEqual( label_recieved , [] )
+        #add one label
+        label_send = ['label_1']
+        transaction = Transaction( self.sessionPath , Transaction.WRITE )
+        transaction.setJobLabels( self.jobID[0] , label_send )
+        transaction.commit()
+        #get one label
+        transaction = Transaction( self.sessionPath , Transaction.READ )
+        label_recieved = transaction.getJobLabels(self.jobID[0])
+        transaction.commit()
+        self.assertEqual( label_recieved , label_send )
+        #add 2 labels
+        label_send = ['label_2', 'label_2_bis']
+        transaction = Transaction( self.sessionPath , Transaction.WRITE )
+        transaction.setJobLabels( self.jobID[1] , label_send )
+        transaction.commit()
+        #get 2 labels
+        transaction = Transaction( self.sessionPath , Transaction.READ )
+        label_recieved = transaction.getJobLabels(self.jobID[1])
+        transaction.commit()
+        self.assertEqual( label_recieved , label_send )
+        #add label in job which have already labels
+        new_label = [ 'label_3', 'label_3_bis' ]
+        transaction = Transaction( self.sessionPath , Transaction.WRITE )
+        transaction.setJobLabels( self.jobID[1] , new_label )
+        transaction.commit()
+        #get 2 new labels
+        transaction = Transaction( self.sessionPath , Transaction.READ )
+        label_recieved = transaction.getJobLabels( self.jobID[1] )
+        label_recieved = transaction.getJobLabels( self.jobID[1] )
+        transaction.commit()
+        self.assertEqual( label_recieved , new_label )
+        
+        
+    def testgetAllUniqueLabels(self):
+        shutil.copy( self.xmlSourcePath , self.sessionPath )
+        label_1= [ 'label_1', 'label_1_bis' ]
+        label_2=  label_1 + [ 'label_2' ]
+        transaction = Transaction( self.sessionPath , Transaction.WRITE )
+        transaction.setJobLabels( self.jobID[0] , label_1 )
+        transaction.setJobLabels( self.jobID[1] , label_2 )
+        transaction.commit()
+        transaction = Transaction( self.sessionPath , Transaction.READ )
+        label_recieved = transaction.getJobLabels( self.jobID[1] )
+        transaction.commit()
+        s = set( label_1 + label_2 )
+        unique = list( s )
+        unique.sort()
+        label_recieved.sort()
+        self.assertEqual( label_recieved , unique )
+        
+    def testSetAndGetJobDescription(self):
+        shutil.copy( self.xmlSourcePath , self.sessionPath )
+        job_description = "a new beautiful description for this job"
+        transaction = Transaction( self.sessionPath , Transaction.WRITE )
+        transaction.setJobDescription( self.jobID[0] , job_description)
+        transaction.commit()
+        transaction = Transaction( self.sessionPath , Transaction.READ )
+        jobDescription_recieved = transaction.getJobDescription( self.jobID[0] )
+        transaction.commit()
+        self.assertEqual( jobDescription_recieved , job_description )
+        transaction = Transaction( self.sessionPath , Transaction.READ )
+        jobDescription_recieved = transaction.getJobDescription( self.jobID[1] )
+        transaction.commit()
+        self.assertEqual( None , jobDescription_recieved )
+        
+        
 if __name__ == '__main__':
     unittest.main()
