@@ -168,13 +168,26 @@ class WorkflowJob(object):
         p = Parameter(mt, name=parameter_name)
         p._isout = wf_parameter.isout
         if dt.isFile():
-            file_name = parameter_name+'.data'
-            if src:
-                src = DataProvider.get(src)
-            file_name, size = mt.toFile( value , self , file_name, src , srcFileName  )
             if not(wf_parameter.isout):
-                self.jobState.addInputDataFile(parameter_name, (file_name, size, None))
+                if dt.isMultiple():
+                    value.sort(key=lambda item:item[4])
+                    for value, file_name, src, srcFileName, index in value:
+                        file_name = parameter_name+'.data'
+                        if src:
+                            src = DataProvider.get(src)
+                        file_name, size = mt.toFile( value , self , file_name, src , srcFileName  )
+                        self.jobState.addInputDataFile(parameter_name, (file_name, size, None))
+                else:
+                    file_name = parameter_name+'.data'
+                    if src:
+                        src = DataProvider.get(src)
+                    file_name, size = mt.toFile( value , self , file_name, src , srcFileName  )
+                    self.jobState.addInputDataFile(parameter_name, (file_name, size, None))
             else:
+                file_name = parameter_name+'.data'
+                if src:
+                    src = DataProvider.get(src)
+                file_name, size = mt.toFile( value , self , file_name, src , srcFileName  )
                 self.jobState.setOutputDataFile(parameter_name, [(file_name, size, None)])
         else:
             if not(wf_parameter.isout):            
@@ -286,28 +299,17 @@ class WorkflowJob(object):
                 msg = "an error occured during the zipping results :\n\n"
                 log.critical( "%s : %s" %( self.id , msg ) , exc_info = True)
                 zipFileName = None
-                
-            if self.email_notify == 'auto':
-                if ( t1 - t0 ) > self.cfg.email_delay() :
-                    emailResults(  self.cfg ,
-                                   self.email, #userEmail, 
-                                   registry, 
-                                   self.id, 
-                                   self.getDir(), 
-                                   self.workflow.getName(),
-                                   self._job.getKey(),  
-                                   FileName = zipFileName )
-                elif self.email_notify == 'true':
-                    emailResults(  self.cfg ,
-                                   self.email, #userEmail, 
-                                   registry, 
-                                   self.id, 
-                                   self.getDir(), 
-                                   self.workflow.getName(),
-                                   self._job.getKey(),  
-                                   FileName = zipFileName )
-                else:
-                    pass    
+
+            if (self.email_notify == 'auto' and ( t1 - t0 ) > self.cfg.email_delay()) or\
+                self.email_notify == 'true':
+                emailResults(  self.cfg ,
+                               self.email, #userEmail, 
+                               registry, 
+                               self.id, 
+                               self.getDir(), 
+                               self.workflow.getName(),
+                               self._job.getKey(),  
+                               FileName = zipFileName )
             sys.exit(0) #exit with no error
         else:
             #return properly to the cgi
@@ -441,7 +443,7 @@ class WorkflowJob(object):
                 if iv.reference is not None:
                     ti['src'] = iv.reference
                     ti['srcFileName'] = iv.safe_name
-                if iv.value is not None:
+                elif iv.value is not None:
                     ti['value'] = iv.value
                 self.data.append(ti)
         for l in self.workflow.links:
@@ -541,9 +543,13 @@ class WorkflowJob(object):
                         o['src'] = output_values[0]['src'] #FIXME WE TAKE ONLY THE FIRST VALUE INTO ACCOUNT!!!
                         o['srcFileName'] = output_values[0]['srcFileName']
                     else:
-                        # if the expected result has not been produced
-                        raise WorkflowJobError("expected output %s has not been produced by task %s (job %s)" 
-                                               %(o['parameter_id'], t.id, job_entry['job_id'])) # error, so nothing to be done
+                        for item in [item for item in self.data if item['type']=='link']:
+                            link = item['link']
+                            if link.to_task is not None and link.from_parameter==o['parameter_id']\
+                              and link.from_task==t.id:
+                                # if the expected result has not been produced
+                                raise WorkflowJobError("expected output %s has not been produced by task %s (job %s)" 
+                                                       %(o['parameter_id'], t.id, job_entry['job_id'])) # error, so nothing to be done
             return True # jobSignal
         else:
             return False
