@@ -33,14 +33,17 @@ def process( self ):
             dt = df.newDataType(datatype_class)
         else:
             dt = df.newDataType(datatype_superclass, datatype_class)
-        mt = MobyleType(dt)
+        
+        dt4session = dt.dataType if dt.isMultiple() else dt
+        mt = MobyleType(dt4session)
         mt.setDataFormat(self.request.getfirst('format'))
+
         try:
             if(fileContent==''):
                 raise UserValueError(msg='empty file, please check your data')
             safeFileName = self.session.addData( fileName , mt, content = fileContent , inputModes = self.request.getfirst("inputMode","upload"))
             contentData = self.session.getContentData(safeFileName)
-            if df.issubclass(mt.getDataType(),"Binary"):
+            if df.issubclass( mt.getDataType() ,"Binary"):
                 self.jsonMap['content'] = "Binary data..."
             else:
                 self.jsonMap['headFlag'] = contentData[0]
@@ -58,6 +61,7 @@ def process( self ):
             self.jsonMap['userName'] = fileName
             self.jsonMap['safeFileName'] = safeFileName
             self.jsonMap['inputModes'] = self.session.getData(safeFileName).get('inputModes')
+            self.jsonMap['sessionUrl'] = self.session.url
 
 if __name__ == "__main__":
     mb_cgi.JSONCGI(processFunction=process,useSession=True)
\ No newline at end of file
