@@ -10,9 +10,14 @@
 #############################################################
 import mb_cgi
 from Mobyle.JobFacade import JobFacade
+from Mobyle.Registry import registry
 
 def process(self):
-    j = JobFacade.getFromJobId(self.request.getfirst('jobId',None))
+    if(self.request.has_key('jobId')):
+        j = JobFacade.getFromJobId(self.request.getfirst('jobId',None))
+    elif(self.request.has_key('jobPid')):
+        pid = registry.getJobURL(self.request.getfirst('jobPid',None))
+        j = JobFacade.getFromJobId(pid)
     self.jsonMap = j.killJob()
     
 if __name__ == "__main__":
